package usecase

import (
	"code.30cm.net/digimon/library-go/errs"
	"code.30cm.net/digimon/library-go/errs/code"
	"context"
	"fmt"
	"github.com/zeromicro/go-zero/core/logx"
	"url_generate/internal/module/url/domain"
	"url_generate/internal/module/url/domain/entity"
	"url_generate/internal/module/url/domain/repository"
	"url_generate/internal/module/url/domain/usecase"
)

const (
	MinEmptyCodeLen = 50
)

type URLUseCaseParam struct {
	URLRepo repository.URLRepository
}

type URLUseCase struct {
	Repo  repository.URLRepository
	IDGen *IDAllocator
}

func (use *URLUseCase) CreateShortCode(ctx context.Context, url string) (string, error) {
	count, err := use.Repo.EmptyCount(ctx)
	if err != nil {
		return "", errs.DatabaseErrorWithScopeL(
			code.CloudEPPortalGW,
			domain.FailedToGetURLEmptyCount,
			logx.WithContext(ctx),
			[]logx.LogField{
				{Key: "url", Value: url},
				{Key: "func", Value: "URLRepo.EmptyCount"},
				{Key: "err", Value: err.Error()},
			},
			"failed to create url short code").Wrap(err)
	}

	// 產生更多的 Code
	if count < MinEmptyCodeLen {
		err := use.GenerateCode(ctx, MinEmptyCodeLen)
		if err != nil {
			return "", errs.SystemInternalError("failed to create url short code")
		}
	}

	shortCode, err := use.Repo.BindOne(ctx, url)
	if err != nil {
		return "", errs.DatabaseErrorWithScopeL(
			code.CloudEPPortalGW,
			domain.FailedToBindShortCode,
			logx.WithContext(ctx),
			[]logx.LogField{
				{Key: "url", Value: url},
				{Key: "func", Value: "URLRepo.BindOne"},
				{Key: "err", Value: err.Error()},
			},
			"failed to create url short code").Wrap(err)
	}

	return shortCode, nil
}

func (use *URLUseCase) GetURLByShortCode(ctx context.Context, shortCode string) (string, error) {
	url, err := use.Repo.FindOne(ctx, shortCode)
	if err != nil {
		return "", errs.DatabaseErrorWithScopeL(
			code.CloudEPPortalGW,
			domain.FailedToFindURLByShortCode,
			logx.WithContext(ctx),
			[]logx.LogField{
				{Key: "shortCode", Value: shortCode},
				{Key: "func", Value: "URLRepo.FindOne"},
				{Key: "err", Value: err.Error()},
			},
			"failed to find url short code").Wrap(err)
	}
	if url.URL == "" {
		return "", errs.ResourceNotFoundWithScope(
			code.CloudEPPortalGW,
			domain.FailedToFindURLByShortCode,
			"failed to find url short code")
	}

	return url.URL, nil
}

func (use *URLUseCase) DeleteShortCode(ctx context.Context, shortCode string) error {
	url, err := use.Repo.FindOne(ctx, shortCode)
	if err != nil {
		return errs.DatabaseErrorWithScopeL(
			code.CloudEPPortalGW,
			domain.FailedToFindURLByShortCode,
			logx.WithContext(ctx),
			[]logx.LogField{
				{Key: "shortCode", Value: shortCode},
				{Key: "func", Value: "URLRepo.FindOne"},
				{Key: "err", Value: err.Error()},
			},
			"failed to find url short code").Wrap(err)
	}
	if url.URL == "" {
		return errs.ResourceNotFoundWithScope(
			code.CloudEPPortalGW,
			domain.FailedToFindURLByShortCode,
			"failed to find url short code")
	}

	err = use.Repo.Update(ctx, shortCode, "")
	if err != nil {
		return errs.DatabaseErrorWithScopeL(
			code.CloudEPPortalGW,
			domain.FailedToFindURLByShortCode,
			logx.WithContext(ctx),
			[]logx.LogField{
				{Key: "shortCode", Value: shortCode},
				{Key: "func", Value: "URLRepo.Delete"},
				{Key: "err", Value: err.Error()},
			},
			"failed to del url short code").Wrap(err)
	}

	// 目前先不考慮錯誤狀況，因為還要插入回去
	_ = use.IDGen.Release(shortCode)

	return nil
}

func (use *URLUseCase) GenerateCode(ctx context.Context, num int) error {
	// 確保請求的生成數量合法
	if num <= 0 {
		return fmt.Errorf("num must be greater than 0")
	}

	// 保存結果的 URLTable 列表
	var result []*entity.URLTable

	// 循環生成指定數量的短碼
	for i := 0; i < num; i++ {
		shortCode, err := use.IDGen.Allocate()
		if err != nil {
			logx.Errorf("Failed to allocate ID: %v\n", err)
			continue // 分配失敗直接跳過
		}

		urlTable := &entity.URLTable{
			ShortCode: shortCode,
		}

		// 插入到結果列表中
		result = append(result, urlTable)
	}

	err := use.Repo.InsertMany(ctx, result)
	if err != nil {
		fmt.Printf("Failed to insert URLTable: %v\n", err)
	}

	return nil
}

func MustURLUseCase(param URLUseCaseParam) usecase.URLUseCase {
	res := &URLUseCase{
		Repo:  param.URLRepo,
		IDGen: NewIDAllocator(),
	}

	return res
}
